@@ -35,6 +35,7 @@
 #define ISINSTANCE(device) dc_device_isinstance((device), &oceanic_atom2_device_vtable.base)
 
 #define VTX        0x4557
+#define I750TC     0x455A
 
 #define MAXRETRIES 2
 #define MAXDELAY   16
@@ -82,6 +83,7 @@ static const oceanic_common_device_vtable_t oceanic_atom2_device_vtable = {
 
 static const oceanic_common_version_t aeris_f10_version[] = {
 	{"FREEWAER \0\0 512K"},
+	{"OCEANF10 \0\0 512K"},
 	{"MUNDIAL R\0\0 512K"},
 };
 
@@ -114,6 +116,7 @@ static const oceanic_common_version_t oceanic_atom2b_version[] = {
 	{"OCEVEO20 \0\0 512K"},
 	{"TUSAZEN \0\0  512K"},
 	{"AQUAI300 \0\0 512K"},
+	{"HOLLDG03 \0\0 512K"},
 };
 
 static const oceanic_common_version_t oceanic_atom2c_version[] = {
@@ -128,7 +131,6 @@ static const oceanic_common_version_t oceanic_default_version[] = {
 	{"ELITET31 \0\0 512K"},
 	{"DATAMASK \0\0 512K"},
 	{"COMPMASK \0\0 512K"},
-	{"HOLLDG03 \0\0 512K"},
 };
 
 static const oceanic_common_version_t oceanic_proplus3_version[] = {
@@ -182,6 +184,7 @@ static const oceanic_common_version_t oceanic_reactpro_version[] = {
 static const oceanic_common_version_t aeris_a300cs_version[] = {
 	{"AER300CS \0\0 2048"},
 	{"OCEANVTX \0\0 2048"},
+	{"AQUAI750 \0\0 2048"},
 };
 
 static const oceanic_common_version_t aqualung_i450t_version[] = {
@@ -594,7 +597,7 @@ oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char
 
 	// Get the correct baudrate.
 	unsigned int baudrate = 38400;
-	if (model == VTX) {
+	if (model == VTX || model == I750TC) {
 		baudrate = 115200;
 	}
 
