@@ -1153,15 +1153,6 @@ uwatec_smart_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 				}
 				sample.gasmix = idx;
 				if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
-#ifdef ENABLE_DEPRECATED
-				unsigned int o2 = parser->gasmix[idx].oxygen;
-				unsigned int he = parser->gasmix[idx].helium;
-				sample.event.type = SAMPLE_EVENT_GASCHANGE2;
-				sample.event.time = 0;
-				sample.event.flags = 0;
-				sample.event.value = o2 | (he << 16);
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
-#endif
 				gasmix_previous = gasmix;
 			}
 
